//---------------------------------------------------------------------------
// jrdebug_output.h
// functions that support output
//---------------------------------------------------------------------------







//---------------------------------------------------------------------------
// formatting workhorse (this calls va_end so caller shouldnt)
void JrDebug::DoVa(va_list &va, const char *format)
	{
	// based originally on code from Paul Mclachlan, http://www.codeproject.com/debug/location_trace.asp#xx386001xx

	// ATTN: we are GUARUNTEED to get here on every debug message? if not, move it up to jrdebug_core.h
	UpdateErrorCounts();

	if (FilterBlock())
		{
		// close the va
		va_end(va);
		// skip processing
		return;
		}

	//  format the user string of the message as requested using arguments: (just the format part)
	char userstring[JrDebug_MaxBufferSize];
	myvsnprintf(userstring, JrDebug_MaxBufferSize-1, format, va);
	userstring[JrDebug_MaxBufferSize-1] = 0;

	// close the va
	va_end(va);

	// make string escape safe
	// ATTN: test
	// EscapeCharp(userstring,JrDebug_MaxBufferSize);

	// Tell JrDebug global object to send this output to all formatters
	DoFormattedOutput(userstring);
	}


void JrDebug::EscapeCharp(char *str,int maxlen)
{
	// escape stuff, ie replace all \n with '\' '\n'
	// this is really messy, just to speed up the case where '\r' not found
	// maybe a better solution would be just check for '\r' or '\n' and if found do a slow safe string replace
	char *outstring = new char [maxlen];
	int zeropos;
	int len=(int)(strlen(str));
	int writepos=0;
	char c;
	int maxwrite=maxlen-2;
	int lastpos=0;
	int count;
	for (count=0;count<len && writepos<maxwrite;++count)
		{
		c=str[count];
		if (c=='\n' || c=='\r')
			{
			if (count==len-1)
				continue;
			if (count>lastpos)
				{
				zeropos=(int)(strlen(outstring)+(count-lastpos));
				if (zeropos>=maxwrite)
					break;
				strncat(outstring,&str[lastpos],count-lastpos);
				outstring[zeropos]='\0';
				lastpos=count+1;
				}
			// change these to \\n
			if (c=='\n')
				strcat(outstring,"\n");
			else if (c=='\r')
				strcat(outstring,"\r");
			}
		}

	if (outstring!=NULL)
		{
		// add end
		zeropos=(int)(strlen(outstring)+(count-lastpos));
		if (zeropos<maxwrite)
			strcat(outstring,&str[lastpos]);
		// copy to original
		strncpy(str,outstring,maxlen);
		str[maxlen]='\0';
		// delete outstring
		delete [] outstring;
		}
}


void JrDebug::DoFormattedOutput(const char *userstring)
	{
	// increment static index of events sent
	++debugindex;

	// if no formatters are instantiated then user f'd up by enabling debugging but not instantiated,
	//  so we create one for them
	if (formatters==NULL)
		{
		// first we try to run default output options
		bool foundcommandlineargs;
		ParseCommandlineArgString(defaultoutputoptions.c_str(),NULL,&foundcommandlineargs);
		// still no formatters?
		if (formatters==NULL)
			ParseCommandlineArgString(JrDebug_DefaultFormatterOptions,NULL,&foundcommandlineargs);
		// still no formatters?
		if (formatters==NULL)
			{
			JrDebugRegisteredOutputFormatter* dbgr=InstantiateFormatter("text","","",true,true);
			if (dbgr==NULL)
				{
				std::cout << "$$$JrDebug Error: Could not instantiate a default output format of 'jrd' type."<<std::endl;
				enabled=false;
				}
			else
				{
				// ATTN: should we warn them that we had to create a default formatter??
				}
			}
		}

	// loop over all instantiated formatters and let them output
	JrDebugRegisteredOutputFormatter* curp=formatters;
	while (curp!=NULL)
		{
		#ifdef _WIN32JRD
		 #ifndef JrDebugDIRECTIVE_DontFixWin32Console
			if (!did_checkwin32console && curp->isStdoutEnabled())
				{
				// first time stdout make sure win32 console is redirected (this will set did_checkwin32console flag)
				Win32RedirectConsoleIfNesc();
				}
		 #endif
		#endif
		curp->HandleDebugMessage(this,userstring);
		curp=curp->get_nextp();
		}
	}
//---------------------------------------------------------------------------





//---------------------------------------------------------------------------
void JrDebug::RegisterFormatterFactory(JrDebugRegisteredOutputFormatter* globalformatterp)
{
	// register the formatter as a factory so we can look it up later
	if (formatter_factorys!=NULL)
		{
	    lastformatter_factoryp->set_nextp(globalformatterp);
		lastformatter_factoryp=globalformatterp;
		}
	else
		{
	    formatter_factorys=globalformatterp;
		lastformatter_factoryp=globalformatterp;
		}
}


void JrDebug::AddInstantiatedFormatter(JrDebugRegisteredOutputFormatter* newp)
{
	// register the formatter as a factory so we can look it up later
	if (formatters!=NULL)
		{
	    lastformatterp->set_nextp(newp);
		lastformatterp=newp;
		}
	else
		{
	    formatters=newp;
		lastformatterp=newp;
		}
}


JrDebugRegisteredOutputFormatter* JrDebug::InstantiateFormatter(const char* formattertypename,const char *optionstring,const char *infilename,bool stdoutflag,bool systemoutflag)
{
	// lookup a registered formatter, instantiate it, and add it to our list
	JrDebugRegisteredOutputFormatter* curp=formatter_factorys;

	// search until we get a name match
	while (curp!=NULL)
		{
		if (curp->doesnamematch(formattertypename))
			{
			// got a match, instantiate it (it will still need to be 'initialized' by our caller
			JrDebugRegisteredOutputFormatter* newp=curp->MakeNew();
			if (newp!=NULL)
				{
				// ok we created it, now initialize and add it
				bool bretv=newp->Initialize(optionstring,infilename,stdoutflag,systemoutflag);
				if (!bretv)
					{
					// delete and return
					delete newp;
					return NULL;
					}
				// successfully initialized
				AddInstantiatedFormatter(newp);
				}
			// and return true
			return newp;
			}
		curp=curp->get_nextp();
		}

	// no match found
	return NULL;
}


void JrDebug::ListFormatters()
{
	// list all registered formatters
	JrDebugRegisteredOutputFormatter* curp=formatter_factorys;

	// announce
	jrd_stdstringT announcement = jrd_stdstringT("listing all output styles");
	// ATTN: do we want to use jrd outputting or stdout?
	//if (!isAnyStdoutEnabled())
		std::cout << std::endl;

	// scoped block announcement
		{
		// create a scoped block object
		// ATTN: do we want to use jrd outputting or stdout?
		// JrDebugBlockObjectData dummy(announcement, __FILE__ , __LINE__ ,  __TIMESTAMP__ , __func__ , true);
		std::cout << announcement <<std::endl;

		// list them all
		while (curp!=NULL)
			{
			// announce, and send output to console no matter what
			announcement = jrd_stdstringT(curp->GetStyleName()) + " - " + curp->GetStyleDescription();
			if (strcmp(curp->GetStyleOptionsDescription(),"")!=0) announcement+= jrd_stdstringT(" (") + curp->GetStyleOptionsDescription() + ").";
			// ATTN: do we want to use jrd outputting or stdout?
			// JrDebug( __FILE__ , __LINE__ ,  __TIMESTAMP__ , __FUNCTION__ , __FUNC__, __func__ , JrdMessage)(JrdInternalInfo,"","",JrDebugDefaultLevel,announcement);
			// send output to console no matter what
			//if (!isAnyStdoutEnabled())
				std::cout << announcement << std::endl;

			curp=curp->get_nextp();
			}
		}
}



void JrDebug::ShutdownFormatters()
	{
	// loop over all instantiated formatters and let them output
	JrDebugRegisteredOutputFormatter* curp=formatters;
	while (curp!=NULL)
		{
		curp->Shutdown();
		curp=curp->get_nextp();
		}
	}



bool JrDebug::isAnyStdoutEnabled()
{
	// helper that returns true if there are any formatters active for stdout
	// useful for commandline info which insists on stdout
	JrDebugRegisteredOutputFormatter* curp=formatters;

	// search until we get a name match
	while (curp!=NULL)
		{
		if (curp->isStdoutEnabled())
			return true;
		curp=curp->get_nextp();
		}
	// none found
	return false;
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
bool JrDebug::FilterBlock()
{
	// are we set to filter out displaying of this one?

	// filter on level
	if (severitylevelnumber<filtermin)
		return true;

	if (filter_onlyerrors)
		{
		if (!filter_showallblocks && !MessageIsSimpleError())
			{
			// ok this is simply, display any errored messages (not including errored blocks
			return true;
			}
		else if (filter_showallblocks && MessageIsError())
			{
			// show errors or block errors
			return false;
			}
		else if (!MessageIsError())
			{
			// otherwise dont show it (ie avoid start blocks)
			return true;
			}
		}

	return false;
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
#ifdef _WIN32JRD
 #ifndef JrDebugDIRECTIVE_DontFixWin32Console
	// for win32 console redirection
	#include <io.h>
	#include <fcntl.h>
	#ifndef JrDebugDIRECTIVE_DontUseGetConsoleWindow
		extern "C" WINBASEAPI HWND WINAPI GetConsoleWindow (); 
	#endif
 #endif
#endif


#ifdef _WIN32JRD
// this only valid for windows
void JrDebug::Win32RedirectConsoleIfNesc()
{
	// win32 apps built with subsystem::windows (instead of subsystems::console) have their stdout send to NULL
	//  and must be redirected

	// this only valid for windows
	#ifndef _WIN32JRD
		did_checkwin32console=true;
		return;
	#endif
	#ifdef JrDebugDIRECTIVE_DontFixWin32Console
		did_checkwin32console=true;
		return;
	#endif

	// already checked?
	if (did_checkwin32console)
		return;
	did_checkwin32console=true;

	// ok set up the redirected console
	HANDLE handle;
	int hCrt=0;

	// set handle
	#ifndef JrDebugDIRECTIVE_DontUseGetConsoleWindow
		// see if one is already set up (GetConsoleWindpw() is good at this)
		handle = GetStdHandle(STD_OUTPUT_HANDLE);		// return an OS file handle
		handle = GetConsoleWindow();
		// do we need to allocate it
		// ATTN: this is not quite working but for now we force it to make a new console
		if (!handle || true)
			{
			AllocConsole();
			did_allocwin32console=true;
			handle = GetStdHandle(STD_OUTPUT_HANDLE);	// return an OS file handle
			}
	#else
		// if we cant use GetConsoleWindow() then we force a new console
		// ATTN: alternativelt we would like to 'guess' whether this win32 app was compiled in console mode
		// in which case we dont need to AllocConsole()
		AllocConsole();
		did_allocwin32console=true;
		handle = GetStdHandle(STD_OUTPUT_HANDLE);		// return an OS file handle
	#endif

	// actually this code should never get here if JrDebugDIRECTIVE_DontFixWin32Console, but this prevents compiler complaints about openosfhandle
	#ifndef JrDebugDIRECTIVE_DontFixWin32Console
		// ATTN: annoying compiler warning (change this to use printf to avoid)
		long lhandle=JrdHandleToLong(handle);

		// redirect stdout
		hCrt = _open_osfhandle(lhandle,_O_TEXT);	// return a runtime file handle
		if (hCrt)
			{
			FILE * hf = _fdopen( hCrt, "w" );					// stream
			if (hf)
				{
				// ok finally we can redirect stdout
				*stdout = *hf;
				}
			}
	#endif

	// turn off buffering of stdout, so output is immediate
	setvbuf (stdout, NULL, _IONBF, 0);
}
#endif


#ifdef _WIN32JRD
long JrDebug::JrdHandleToLong(HANDLE hnd)
{
	// this is a allowable but unfortunate "pointer truncation" warning that we want to bypass
	// note that this is called on startup if needed, only once, so speed is not an issue
	long lretv;

	// we could just do (but we get a warning)
	lretv=(long)hnd;

	return lretv;
}
#endif
//---------------------------------------------------------------------------























//---------------------------------------------------------------------------
JrDebugRegisteredOutputFormatter::JrDebugRegisteredOutputFormatter()
{
	// nothing really to do here, main initialization is done in Initialize
	// we just clear filepointer so its safe for deletion
	filepointer=NULL;
	linesout=0;
	nextp=NULL;
}


JrDebugRegisteredOutputFormatter::JrDebugRegisteredOutputFormatter(bool isfactory)
	{
	// nothing really to do here, main initialization is done in Initialize
	// touch unused args to avoid compiler warnings
	JrdUnusedArg(isfactory);
	}

JrDebugRegisteredOutputFormatter::~JrDebugRegisteredOutputFormatter()
	{
	// close any existing file?
	if (filepointer!=NULL)
		{
		fclose(filepointer);
		filepointer=NULL;
		}
	}
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
bool JrDebugRegisteredOutputFormatter::doesnamematch(const char* formattertypename)
{
	// return true if name matches name
	// ATTN: should this be made case insensitive?
	if (strcmp(formattertypename,GetStyleName())==0)
		return true;
	return false;
}
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
bool JrDebugRegisteredOutputFormatter::Initialize(const char *optionstring,const char *infilename,bool stdoutflag,bool systemoutflag)
{
	// initialize
	enabled_stdout=stdoutflag;;
	enabled_systemout=systemoutflag;
	enabled_fileout=false;

	// file info
	filename=jrd_stdstringT(infilename);
	filename=ReplaceFileNameArgs(filename);

	// if no extension, add default extension
	if (filename!="" && filename.find(".")==std::string::npos)
		{
		filename+=jrd_stdstringT(".")+GetDefaultFileExt();
		}

	// ATTN: pass in appendmode option?
	appendmode=false;

	// ATTN: should we try to open the file for output now, rather than on write? i think so
	if (filename!="")
		CreateOutputFile(filename.c_str(),appendmode);

	// call virtual startup code
	bool bretv = Startup(optionstring);

	// return success
	return bretv;
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
jrd_stdstringT JrDebugRegisteredOutputFormatter::GetHeaderAnnounceString()
{
	// just return a string with some version info
	jrd_stdstringT str;
	// current time
	time_t timenum;
	time(&timenum);
	jrd_stdstringT datestring = jrd_stdstringT(ctime(&timenum));
	// result
	str = jrd_stdstringT("JrDebug - version ")+jrd_stdstringT(JrDebugVersionString)+" - "+datestring+" - "+JrDebug::get_programdescription();
	return str;
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
jrd_stdstringT JrDebugRegisteredOutputFormatter::ReplaceFileNameArgs(jrd_stdstringT &infname)
{
	// this is a STATIC function
	jrd_stdstringT filename;
	time_t timenum;
	time(&timenum);
	tm *curtmp;

	curtmp = localtime(&timenum);

	// call strftime(char *s, size_t maxsize, const char *format, const struct tm *timptr);
	char resultbuf[255];
	strftime(resultbuf,254,infname.c_str(),curtmp);
	resultbuf[254]='\0';

		/*
		$###    is replaced by 001 , 002 , etc. until a unique filename is found (specify as many # as you want)
		.
		. FROM MS DOCS (subset of below, so use these)
		%a 		Abbreviated weekday name 
		%A 		Full weekday name 
		%b 		Abbreviated month name 
		%B 		Full month name 
		%c 		Date and time representation appropriate for locale 
		%d 		Day of month as decimal number (01  31) 
		%H 		Hour in 24-hour format (00  23) 
		%I 		Hour in 12-hour format (01  12) 
		%j 		Day of year as decimal number (001  366) 
		%m 		Month as decimal number (01  12) 
		%M 		Minute as decimal number (00  59) 
		%p 		Current locale's A.M./P.M. indicator for 12-hour clock 
		%S 		Second as decimal number (00  59) 
		%U 		Week of year as decimal number, with Sunday as first day of week (00  53) 
		%w 		Weekday as decimal number (0  6; Sunday is 0) 
		%W 		Week of year as decimal number, with Monday as first day of week (00  53) 
		%x 		Date representation for current locale 
		%X 		Time representation for current locale 
		%y 		Year without century, as decimal number (00  99) 
		%Y 		Year with century, as decimal number 
		%z,%Z 	Either the time-zone name or time zone abbreviation, depending on registry settings; no characters if time zone is unknown 
		%% 		Percent sign 
		.
		. FROM UNIX DOCS:
		%a		is replaced by the locale's abbreviated weekday name. 
		%A		is replaced by the locale's full weekday name. 
		%b		is replaced by the locale's abbreviated month name. 
		%B		is replaced by the locale's full month name. 
		%c		is replaced by the locale's appropriate date and time representation. 
		%C		is replaced by the century number (the year divided by 100 and truncated to an integer) as a decimal number [00-99]. 
		%d		is replaced by the day of the month as a decimal number [01,31]. 
		%D		same as %m/%d/%y. 
		%e		is replaced by the day of the month as a decimal number [1,31]; a single digit is preceded by a space. 
		%h		same as %b. 
		%H		is replaced by the hour (24-hour clock) as a decimal number [00,23]. 
		%I		is replaced by the hour (12-hour clock) as a decimal number [01,12]. 
		%j		is replaced by the day of the year as a decimal number [001,366]. 
		%m		is replaced by the month as a decimal number [01,12]. 
		%M		is replaced by the minute as a decimal number [00,59]. 
		%n		is replaced by a newline character. 
		%p		is replaced by the locale's equivalent of either a.m. or p.m. 
		%r		is replaced by the time in a.m. and p.m. notation; in the POSIX locale this is equivalent to %I:%M:%S %p. 
		%R		is replaced by the time in 24 hour notation (%H:%M). 
		%S		is replaced by the second as a decimal number [00,61]. 
		%t		is replaced by a tab character. 
		%T		is replaced by the time (%H:%M:%S). 
		%u		is replaced by the weekday as a decimal number [1,7], with 1 representing Monday. 
		%U		is replaced by the week number of the year (Sunday as the first day of the week) as a decimal number [00,53]. 
		%V		is replaced by the week number of the year (Monday as the first day of the week) as a decimal number [01,53]. If the week containing 1 January has four or more days in the new year, then it is considered week 1. Otherwise, it is the last week of the previous year, and the next week is week 1. 
		%w		is replaced by the weekday as a decimal number [0,6], with 0 representing Sunday. 
		%W		is replaced by the week number of the year (Monday as the first day of the week) as a decimal number [00,53]. All days in a new year preceding the first Monday are considered to be in week 0. 
		%x		is replaced by the locale's appropriate date representation. 
		%X		is replaced by the locale's appropriate time representation. 
		%y		is replaced by the year without century as a decimal number [00,99]. 
		%Y		is replaced by the year with century as a decimal number. 
		%Z		is replaced by the timezone name or abbreviation, or by no bytes if no timezone information exists. 
		%%		is replaced by %. 
		*/

	// now we do our custom replacement of $### to unique filename
	char *spos,*epos;
	int numlen;
	spos=strstr(resultbuf,"$#");
	if (spos!=NULL)
		{
		// get the number of # characters
		epos=spos+2;
		while (*epos=='#')
			++epos;
		numlen=(int)(epos-spos)-1;
		if (numlen>100)
			numlen=100;
		// copy over the $
		strcpy(spos,(char*)(spos+1));
		// now loop and try all possible file names
		long nlimit=1;
		for (int count=0;count<numlen;++count)
			nlimit*=10;
		--nlimit;
		char numstr[101];
		FILE *filep;
                long lcount;
		for (lcount=1;lcount<nlimit;++lcount)
			{
			// form 0001 num
			sprintf(numstr,"%0*ld",numlen,lcount);
			// copy it in without the '\0'
			strncpy(spos,numstr,numlen);
			// does file exist?
			if ((filep=fopen(resultbuf,"rb"))==NULL)
				{
				// got a unique one
				break;
				}
			fclose(filep);
			}
		if (lcount>=nlimit)
			{
			// couldnt find a unique filename using ##
			}
		}

	// assign filename
	filename=jrd_stdstringT(resultbuf);

	return filename;
}
//---------------------------------------------------------------------------
























//---------------------------------------------------------------------------
// Send line to file/stdout/sytem debug message
void JrDebugRegisteredOutputFormatter::DoOutput(const char *str)
	{
	// show to stdout console?
	if (enabled_stdout)
		printf("%s",str);
	
	// output to system debug pipe/debugtrace?
	if (enabled_systemout)
		OutputDebugStringDoOutput(str);

	// output to file?
	if (filepointer!=NULL && enabled_fileout)
		{
		fprintf(filepointer,"%s",str);
		// force flush right away
		fflush(filepointer);
		}

	// keep track of lines written
	++linesout;
	}
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// Send line to file/stdout/sytem debug message, helper for indenting etc
void JrDebugRegisteredOutputFormatter::DoOutput(const char *str, int repetitions)
	{
	int count;

	// show to stdout console?
	if (enabled_stdout)
		{
		for (count=0;count<repetitions;++count)
			printf("%s",str);
		}

	// output to system debug pipe/debugtrace?
	if (enabled_systemout)
		{
		for (count=0;count<repetitions;++count)
			OutputDebugStringDoOutput(str);
		}

	// output to file?
	if (filepointer!=NULL && enabled_fileout)
		{
		for (count=0;count<repetitions;++count)
			fprintf(filepointer,"%s",str);
		}

	// keep track of lines written
	++linesout;
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
bool JrDebugRegisteredOutputFormatter::CreateOutputFile(const char *fname,bool appendmode)
	{
	// open global file
	jrd_stdstringT rfilename=jrd_stdstringT(fname);

	// 6/29/08 - auto append mode from filename
	int len = rfilename.length();
	if (len>2 && rfilename[len-2]=='-' && rfilename[len-1]=='a')
		{
		appendmode = true;
		rfilename=rfilename.substr(0,len-2);
		}



	// ATTN: fixup filename for vista or win7?
			#ifdef _WIN32JRD
				// environment expansion done prior
				//char tempoutbuf[2056];
				//ExpandEnvironmentStrings(filename.c_str(), tempoutbuf, 2055);
				//filename=jrd_stdstringT(tempoutbuf);
				// ATTN: smart windows version replacement
				// if run on vista (or win7?) the application will not be able to write to its own directory (STUPID IDIOTIC MS DECISION)
				//  so we should fix this up
			#endif


	if (rfilename=="")
		rfilename="jrdebugout.log";
	// returns true on success
	if (filepointer!=NULL)
		fclose(filepointer);
	if (appendmode)
		filepointer=fopen(rfilename.c_str(),"a");
	else
		filepointer=fopen(rfilename.c_str(),"w");
	if (!filepointer)
		{
		enabled_fileout=false;
		return false;
		}
	else
		{
		enabled_fileout=true;
		// line divider if appending
		if (appendmode)
			fputs("<!-- ------------------------------------------------------------------------ -->\n",filepointer);
		}
	return true;
	}
//---------------------------------------------------------------------------


































//---------------------------------------------------------------------------
// shutdown and startup
bool JrDebugRegisteredOutputFormatter::Startup(const char *optionstring)
{
	// virtual startup
	// touch unused args to avoid compiler warnings
	JrdUnusedArg(optionstring);
	return true;
}

bool JrDebugRegisteredOutputFormatter::Shutdown()
{
	// virtual shut down (any open files will be closed by caller when we return)
	return true;
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// HandleDebugMessage should format the user's string and call DoOutput one or more times
// Note it will be invoked with any user % args already repalced
void JrDebugRegisteredOutputFormatter::HandleDebugMessage(JrDebug *jrd,const char *str)
{
	// by default just send it directly to output
	DoOutput(str);
	// touch unused args to avoid compiler warnings
	JrdUnusedArg(jrd);
}
//---------------------------------------------------------------------------

