//---------------------------------------------------------------------------
// jrdebug_output_formatters_compiler.h
// derived methods for different outputs
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Compiler (for output emulating output of gcc,bcc,msvc compilers)
// register the factory
JrDebugRegisterOutputFormatter(JrDebugRegisteredOutputFormatter_Compiler);
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// constructor and destructor
JrDebugRegisteredOutputFormatter_Compiler::JrDebugRegisteredOutputFormatter_Compiler(bool isfactory)
	:JrDebugRegisteredOutputFormatter()
{
	// register if appropraite
	if (isfactory)
		{
		// create a NEW object for registering, dont try to attach this global one
		JrDebugSingletonInsurer dummy(false);
		JrDebug::RegisterFormatterFactory(MakeNew());
		return;
		}
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// shutdown and startup
bool JrDebugRegisteredOutputFormatter_Compiler::Startup(const char *optionstring)
{
	// virtual startup

	// process optionstring
	if (strcmp(optionstring,"")==0)
		emulationmode=JrDebug_CompilerOutput_EmulationMode_Generic;
	else if (strcmp(optionstring,"gcc")==0)
		emulationmode=JrDebug_CompilerOutput_EmulationMode_Gcc;
	else if (strcmp(optionstring,"bcc")==0)
		emulationmode=JrDebug_CompilerOutput_EmulationMode_Bcc;
	else if (strcmp(optionstring,"msvc")==0)
		emulationmode=JrDebug_CompilerOutput_EmulationMode_Msvc;
	else if (strcmp(optionstring,"generic")==0)
		emulationmode=JrDebug_CompilerOutput_EmulationMode_Generic;
	else
		{
		// error, not understood option
		return false;
		}

	return true;
}

bool JrDebugRegisteredOutputFormatter_Compiler::Shutdown()
{
	// virtual shut down (any open files will be closed by caller when we return)
	return true;
}
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// HandleDebugMessage should format the user's string and call DoOutput one or more times
// Note it will be invoked with any user % args already repalced
void JrDebugRegisteredOutputFormatter_Compiler::HandleDebugMessage(JrDebug *jrd,const char *str)
{
	// Compiler output emulation

	/// From the open source project CppUnit (cpptest-compileroutput.h), 2/17/05
	/// Generic compiler format, which equals:
	/// <tt>%%file:%%line: %%text</tt>
	/// 
	/// <a href="http://www.borland.com/products/downloads/download_cbuilder.html">
	/// Borland C++ Compiler</a> (BCC) format, which equals:
	/// <tt>Error cpptest %%file %%line: %%text</tt>.
	///
	/// <a href="http://gcc.gnu.org">GNU Compiler Collection</a> 
	/// (GCC) format, which equals:
	/// <tt>%%file:%%line: %%text</tt>
	///
	/// <a href="http://www.microsoft.com">Microsoft Visual C++</a> 
	/// (MSVC) format, which equals:
	/// <tt>%%file(%%line) : %%text</tt>
	///
	///		"%file:%line: %text", 					// Generic
	///		"Error cpptest %file %line: %text",		// BCC
	///		"%file:%line: %text", 					// GCC
	///		"%file(%line) : %text"					// MSVC
	///

	// Format for plaintext, just send it directly but add newline
	char strout[JrDebug_MaxBufferSize];
	jrd_stdstringT filenamestring = jrd->get_filenamestring();
	long linenumber=jrd->get_linenumber();

	// depending if snprintf is available
	if (emulationmode==JrDebug_CompilerOutput_EmulationMode_Gcc)
		mysnprintf(strout, JrDebug_MaxBufferSize-1,"%s:%ld: %s\n",filenamestring.c_str(),linenumber,str);
	else if (emulationmode==JrDebug_CompilerOutput_EmulationMode_Bcc)
		mysnprintf(strout, JrDebug_MaxBufferSize-1,"Error jrdebug %s %ld: %s\n",filenamestring.c_str(),linenumber,str);
	else if (emulationmode==JrDebug_CompilerOutput_EmulationMode_Msvc)
		mysnprintf(strout, JrDebug_MaxBufferSize-1,"%s(%ld): %s\n",filenamestring.c_str(),linenumber,str);
	else if (emulationmode==JrDebug_CompilerOutput_EmulationMode_Generic)
		mysnprintf(strout, JrDebug_MaxBufferSize-1,"%s(%ld) : %s\n",filenamestring.c_str(),linenumber,str);
	strout[JrDebug_MaxBufferSize-1]='\0';

	DoOutput(strout);
}
//---------------------------------------------------------------------------
