//---------------------------------------------------------------------------
// jrdebug_output_formatters_xml.h
// derived methods for different outputs
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Csv (comma separated fields)
// register the factory
JrDebugRegisterOutputFormatter(JrDebugRegisteredOutputFormatter_Xml);
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// constructor and destructor
JrDebugRegisteredOutputFormatter_Xml::JrDebugRegisteredOutputFormatter_Xml(bool isfactory)
:JrDebugRegisteredOutputFormatter()
{
	// register if appropraite
	if (isfactory)
	{
		// create a NEW object for registering, dont try to attach this global one
		JrDebugSingletonInsurer dummy(false);
		JrDebug::RegisterFormatterFactory(MakeNew());
		return;
	}
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// shutdown and startup
bool JrDebugRegisteredOutputFormatter_Xml::Startup(const char *optionstring)
{
	// process optionstring
	if (strcmp(optionstring,"")!=0)
		return false;

	// header
	DoOutput("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n");
	DoOutput("<!-- ");
	DoOutput((GetHeaderAnnounceString()).c_str());
	DoOutput(" -->\n\n");

	// return success
	return true;
}

bool JrDebugRegisteredOutputFormatter_Xml::Shutdown()
{
	// footer

	// return success
	return true;
}
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// HandleDebugMessage should format the user's string and call DoOutput one or more times
// Note it will be invoked with any user % args already replaced
void JrDebugRegisteredOutputFormatter_Xml::HandleDebugMessage(JrDebug *jrd,const char *str)
{
	char strout[JrDebug_MaxBufferSize];

	// function name is present?
	jrd_stdstringT functionnamestring = jrd->get_functionnamestring();
	// file info
	jrd_stdstringT filedirstring = jrd->get_filedirstring();
	jrd_stdstringT filenamestring = jrd->get_filenamestring();
	// misc
	jrd_stdstringT messagetypestring = jrd->get_messagetypestring();
	jrd_stdstringT timestampstring = jrd->get_timestampstring();
	long linenumber=jrd->get_linenumber();
	long runid=jrd->get_runid();
	int severitylevelnumber=jrd->get_severitylevelnumber();
	long debugindex=jrd->get_debugindex();
	// get current time
	time_t timenum;
	time(&timenum);
	// type
	JrdType jrdtype = jrd->get_messagetype();
	// depth, for nice indenting
	int ourdepth=jrd->get_depth();
	char indentstr[8];strcpy(indentstr,"  ");

	// block start
	if (jrdtype==JrdBlockStart)
		{
		DoOutput(indentstr,ourdepth);
		DoOutput("<block>\n");
		}

	// construct the format string to display (uses snprintf for safety to insure we dont go over buffer size)
	DoOutput(indentstr,ourdepth); mysnprintf(strout, JrDebug_MaxBufferSize-1, "<msg>\n"); strout[JrDebug_MaxBufferSize-1] = 0; DoOutput(strout);
	DoOutput(indentstr,ourdepth); mysnprintf(strout, JrDebug_MaxBufferSize-1, " <runid>%ld</runid>\n",runid); strout[JrDebug_MaxBufferSize-1] = 0; DoOutput(strout);
	DoOutput(indentstr,ourdepth); mysnprintf(strout, JrDebug_MaxBufferSize-1, " <time>%ld</time>\n",(long)timenum); strout[JrDebug_MaxBufferSize-1] = 0; DoOutput(strout);
	DoOutput(indentstr,ourdepth); mysnprintf(strout, JrDebug_MaxBufferSize-1, " <index>%ld</index>\n",debugindex); strout[JrDebug_MaxBufferSize-1] = 0; DoOutput(strout);
	DoOutput(indentstr,ourdepth); mysnprintf(strout, JrDebug_MaxBufferSize-1, " <type>%s</type>\n",messagetypestring.c_str()); strout[JrDebug_MaxBufferSize-1] = 0; DoOutput(strout);
	DoOutput(indentstr,ourdepth); mysnprintf(strout, JrDebug_MaxBufferSize-1, " <level>%d</level>\n",severitylevelnumber); strout[JrDebug_MaxBufferSize-1] = 0; DoOutput(strout);
	DoOutput(indentstr,ourdepth); mysnprintf(strout, JrDebug_MaxBufferSize-1, " <file>\"%s\"</file>\n",filenamestring.c_str()); strout[JrDebug_MaxBufferSize-1] = 0; DoOutput(strout);
	DoOutput(indentstr,ourdepth); mysnprintf(strout, JrDebug_MaxBufferSize-1, " <filepath>\"%s\"</filepath>\n",filedirstring.c_str()); strout[JrDebug_MaxBufferSize-1] = 0; DoOutput(strout);
	DoOutput(indentstr,ourdepth); mysnprintf(strout, JrDebug_MaxBufferSize-1, " <timestamp>%s</timestamp>\n", timestampstring.c_str()); strout[JrDebug_MaxBufferSize-1] = 0; DoOutput(strout);
	DoOutput(indentstr,ourdepth); mysnprintf(strout, JrDebug_MaxBufferSize-1, " <line>%ld</line>\n",linenumber); strout[JrDebug_MaxBufferSize-1] = 0; DoOutput(strout);
	DoOutput(indentstr,ourdepth); mysnprintf(strout, JrDebug_MaxBufferSize-1, " <function>%s</function>\n",functionnamestring.c_str()); strout[JrDebug_MaxBufferSize-1] = 0; DoOutput(strout);
	DoOutput(indentstr,ourdepth); mysnprintf(strout, JrDebug_MaxBufferSize-1, " <text>%s</text>\n",str); strout[JrDebug_MaxBufferSize-1] = 0; DoOutput(strout);
	DoOutput(indentstr,ourdepth); mysnprintf(strout, JrDebug_MaxBufferSize-1, "</msg>\n"); strout[JrDebug_MaxBufferSize-1] = 0; DoOutput(strout);

	// block end
	if (jrdtype==JrdBlockEnd || jrdtype==JrdBlockEndError)
	{
		DoOutput(indentstr,ourdepth);
		DoOutput("</block>\n");
	}
}
//---------------------------------------------------------------------------
